import { K as KeyManagementAlgorithm, d as ContentEncryptionAlgorithm, f as JWK_EC_Public, m as JWK } from './unjwt.D9da8HQF.mjs';
import { J as JoseHeaderParameters, a as JWTClaims } from './unjwt.BvIkEGGz.mjs';

/** Recognized JWE Header Parameters, any other Header Members may also be present. */
interface JWEHeaderParameters extends JoseHeaderParameters {
    /**
     * `alg` (Algorithm): Header Parameter - Key Management Algorithm
     */
    alg?: string;
    /**
     * `enc` (Encryption Algorithm): Header Parameter - Content Encryption Algorithm
     */
    enc?: string;
    /**
     * `p2c` (PBES2 Count): Header Parameter
     */
    p2c?: number;
    /**
     * `p2s` (PBES2 Salt): Header Parameter. Base64URL encoded.
     */
    p2s?: string;
    /**
     * "iv" (Initialization Vector) Header Parameter for AES GCM Key Wrapping. Base64URL encoded.
     */
    iv?: string;
    /**
     * "tag" (Authentication Tag) Header Parameter for AES GCM Key Wrapping. Base64URL encoded.
     */
    tag?: string;
    /**
     * "epk" (Ephemeral Public Key) Header Parameter for ECDH-ES.
     */
    epk?: JWK_EC_Public;
    /**
     * "apu" (Agreement PartyUInfo) Header Parameter for ECDH-ES. Base64URL encoded.
     */
    apu?: string;
    /**
     * "apv" (Agreement PartyVInfo) Header Parameter for ECDH-ES. Base64URL encoded.
     */
    apv?: string;
    /** Any other JWE Header member. */
    [propName: string]: unknown;
}
/** Recognized JWE Key Management-related Header Parameters. */
interface JWEKeyManagementHeaderParameters {
    apu?: Uint8Array;
    apv?: Uint8Array;
    /**
     * For Internal use only
     */
    p2c?: number;
    /**
     * For Internal use only
     */
    p2s?: Uint8Array;
    /**
     * For Internal use only
     */
    iv?: Uint8Array;
    /**
     * For Internal use only
     */
    epk?: CryptoKey;
}
/**
 * JWE (JSON Web Encryption) encryption options
 */
interface JWEEncryptOptions {
    /** JWE "alg" (Algorithm) Header Parameter. Key Management Algorithm. */
    alg?: KeyManagementAlgorithm;
    /** JWE "enc" (Encryption Algorithm) Header Parameter. Content Encryption Algorithm. */
    enc?: ContentEncryptionAlgorithm;
    /** Additional JWE Protected Header parameters. */
    protectedHeader?: Omit<JWEHeaderParameters, "alg" | "enc" | "iv" | "tag" | "p2s" | "p2c" | "apu" | "apv">;
    /** Initialization Vector for AES-GCMKW key wrapping. Generated if not provided. */
    keyManagementIV?: Uint8Array;
    /** PBES2 Salt value (p2s). Required for PBES2 algorithms. */
    p2s?: Uint8Array;
    /** PBES2 Iteration count (p2c). Required for PBES2 algorithms. */
    p2c?: number;
    /** ECDH-ES Agreement PartyUInfo. */
    ecdhPartyUInfo?: Uint8Array;
    /** ECDH-ES Agreement PartyVInfo. */
    ecdhPartyVInfo?: Uint8Array;
    /**
     * Content Encryption Key (CEK) to use.
     * If provided, it will be used directly. Otherwise, a CEK will be generated.
     */
    cek?: Uint8Array;
    /**
     * Initialization Vector for content encryption.
     * If provided, it will be used. Otherwise, one will be generated.
     */
    contentEncryptionIV?: Uint8Array;
}
/**
 * Key lookup function for JWE decryption.
 * @param header The JWE Protected Header.
 * @returns The key material (CryptoKey, JWK, password string, or raw Uint8Array) or a Promise resolving to it.
 */
type JWEKeyLookupFunction = (header: JWEHeaderParameters, token: string) => CryptoKey | JWK | string | Uint8Array | Promise<CryptoKey | JWK | string | Uint8Array>;
/**
 * JWE (JSON Web Encryption) decryption options
 */
interface JWEDecryptOptions {
    /** A list of allowed JWE "alg" (Algorithm) Header Parameter values for key management. */
    algorithms?: KeyManagementAlgorithm[];
    /** A list of allowed JWE "enc" (Encryption Algorithm) Header Parameter values for content encryption. */
    encryptionAlgorithms?: ContentEncryptionAlgorithm[];
    /**
     * Critical Header Parameters to be understood and processed.
     * If the JWE contains critical headers not in this list (and not inherently understood by the library), decryption will fail.
     */
    critical?: string[];
    /** Algorithm to import the unwrapped CEK as (e.g., { name: 'AES-GCM' }). Defaults based on 'enc'. */
    unwrappedKeyAlgorithm?: Parameters<typeof crypto.subtle.importKey>[2];
    /** Key usages for the unwrapped CEK. Defaults based on 'enc' (typically ['encrypt', 'decrypt']). */
    keyUsage?: KeyUsage[];
    /** Mark the unwrapped CEK as extractable. Defaults to true. */
    extractable?: boolean;
    /** If true, forces the payload to be returned as a Uint8Array, otherwise type is inferred based on JWE headers. */
    forceUint8Array?: boolean;
}
/**
 * Result of a JWE decryption operation.
 */
interface JWEDecryptResult<T extends JWTClaims | Uint8Array | string = JWTClaims | Uint8Array | string> {
    /** The decrypted payload. */
    payload: T;
    /** The JWE Protected Header. */
    protectedHeader: JWEHeaderParameters;
    /** The Content Encryption Key (CEK) used for decryption, as Uint8Array. */
    cek: Uint8Array;
    /** The Additional Authenticated Data (AAD) used, as Uint8Array. */
    aad: Uint8Array;
}

/**
 * Encrypts a payload to produce a JWE Compact Serialization string.
 *
 * @param payload The payload to encrypt. Can be a string, Uint8Array, or a JSON-serializable object.
 * @param key The key encryption key (KEK). Can be a CryptoKey, JWK, or for PBES2, a password string/Uint8Array.
 * @param options JWE encryption options, including `alg` (key management algorithm) and `enc` (content encryption algorithm).
 * @returns A Promise resolving to the JWE Compact Serialization string.
 */
declare function encrypt(payload: JWTClaims, key: JWK | string | Uint8Array, options?: JWEEncryptOptions): Promise<string>;
declare function encrypt(payload: string | Uint8Array | Record<string, any>, key: JWK | string | Uint8Array, options?: JWEEncryptOptions): Promise<string>;
declare function encrypt(payload: JWTClaims, key: CryptoKey, options: JWEEncryptOptions & {
    alg: KeyManagementAlgorithm;
    enc: ContentEncryptionAlgorithm;
}): Promise<string>;
declare function encrypt(payload: string | Uint8Array | Record<string, any>, key: CryptoKey, options: JWEEncryptOptions & {
    alg: KeyManagementAlgorithm;
    enc: ContentEncryptionAlgorithm;
}): Promise<string>;
declare function encrypt(payload: string | Uint8Array | Record<string, any>, key: CryptoKey | JWK | string | Uint8Array, options: JWEEncryptOptions & {
    alg: KeyManagementAlgorithm;
    enc: ContentEncryptionAlgorithm;
}): Promise<string>;
/**
 * Decrypts a JWE Compact Serialization string.
 *
 * @param jwe The JWE Compact Serialization string.
 * @param key The key decryption key (KEK) or a function to look up the key.
 * @param options JWE decryption options.
 * @returns A Promise resolving to an object containing the decrypted plaintext, protected header, CEK, and AAD.
 * @throws If JWE is invalid, decryption fails, or options are not met.
 */
declare function decrypt<T extends JWTClaims | Uint8Array | string>(jwe: string, key: CryptoKey | JWK | string | Uint8Array | JWEKeyLookupFunction, options?: JWEDecryptOptions): Promise<JWEDecryptResult<T>>;
declare function decrypt(jwe: string, key: CryptoKey | JWK | string | Uint8Array | JWEKeyLookupFunction, options: JWEDecryptOptions & {
    forceUint8Array: true;
}): Promise<JWEDecryptResult<Uint8Array>>;

type jwe_JWEDecryptOptions = JWEDecryptOptions;
type jwe_JWEDecryptResult<T extends JWTClaims | Uint8Array | string = JWTClaims | Uint8Array | string> = JWEDecryptResult<T>;
type jwe_JWEEncryptOptions = JWEEncryptOptions;
type jwe_JWEHeaderParameters = JWEHeaderParameters;
type jwe_JWEKeyLookupFunction = JWEKeyLookupFunction;
type jwe_JWEKeyManagementHeaderParameters = JWEKeyManagementHeaderParameters;
declare const jwe_decrypt: typeof decrypt;
declare const jwe_encrypt: typeof encrypt;
declare namespace jwe {
  export { jwe_decrypt as decrypt, jwe_encrypt as encrypt };
  export type { jwe_JWEDecryptOptions as JWEDecryptOptions, jwe_JWEDecryptResult as JWEDecryptResult, jwe_JWEEncryptOptions as JWEEncryptOptions, jwe_JWEHeaderParameters as JWEHeaderParameters, jwe_JWEKeyLookupFunction as JWEKeyLookupFunction, jwe_JWEKeyManagementHeaderParameters as JWEKeyManagementHeaderParameters };
}

export { encrypt as f, decrypt as g, jwe as j };
export type { JWEHeaderParameters as J, JWEKeyManagementHeaderParameters as a, JWEEncryptOptions as b, JWEKeyLookupFunction as c, JWEDecryptOptions as d, JWEDecryptResult as e };
