import { G as GenerateKeyAlgorithm, a as GenerateKeyOptions, b as GenerateKeyReturn, x as JWK_PBES2, D as DeriveKeyOptions, c as DeriveKeyReturn, l as JWK_oct, m as JWK, K as KeyManagementAlgorithm, W as WrapKeyOptions, e as WrapKeyResult, U as UnwrapKeyOptions, F as JWKPEMAlgorithm, n as JWKSet, C as CompositeKey, d as ContentEncryptionAlgorithm, E as JWKAlgorithm, J as JWKParameters, v as JWK_AES_CBC_HMAC, w as JWK_AES_GCM, u as JWK_AES_GCM_KW, t as JWK_AES_KW, B as JWK_Asymmetric_Algorithm, h as JWK_EC, y as JWK_ECDH_ES, r as JWK_ECDSA, g as JWK_EC_Private, f as JWK_EC_Public, z as JWK_ED, o as JWK_HMAC, k as JWK_RSA, s as JWK_RSA_ENC, q as JWK_RSA_PSS, j as JWK_RSA_Private, i as JWK_RSA_Public, p as JWK_RSA_SIGN, A as JWK_Symmetric_Algorithm } from './unjwt.D9da8HQF.mjs';
import { J as JoseHeaderParameters } from './unjwt.BvIkEGGz.mjs';

/**
 * This is a fork of Jose library's utility functions.
 * @source https://github.com/panva/jose/tree/69b7960c67e05be55fa2ec31c74b987696c20c60/src/lib/asn1.ts
 * @license MIT https://github.com/panva/jose/blob/69b7960c67e05be55fa2ec31c74b987696c20c60/LICENSE.md
 */
/** Key Import Function options. */
interface KeyImportOptions {
    /**
     * The value to use as {@link !SubtleCrypto.importKey} `extractable` argument. Default is false for
     * private keys, true otherwise.
     */
    extractable?: boolean;
}

/**
 * Generates a cryptographic key for the specified algorithm.
 *
 * @param alg The JWA algorithm identifier (e.g., "HS256", "RS256", "A128GCM").
 * @param options Configuration options for key generation.
 * @returns A Promise resolving to the generated key (CryptoKey, CryptoKeyPair, or Uint8Array) or its JWK representation.
 */
declare function generateKey<TAlg extends GenerateKeyAlgorithm, TOptions extends GenerateKeyOptions>(alg: TAlg, options?: TOptions): Promise<GenerateKeyReturn<TAlg, TOptions>>;
/**
 * Derives a key from a password using PBKDF2 as specified by PBES2 algorithms.
 *
 * @param password The password to derive the key from (string or Uint8Array).
 * @param alg The PBES2 algorithm identifier (e.g., "PBES2-HS256+A128KW").
 * @param options Configuration options including salt and iterations.
 * @returns A Promise resolving to the derived key (CryptoKey) or its JWK_oct representation.
 */
declare function deriveKeyFromPassword<TAlg extends JWK_PBES2, TOptions extends DeriveKeyOptions>(password: string | Uint8Array, alg: TAlg, options: TOptions): Promise<DeriveKeyReturn<TOptions>>;
/**
 * Imports a key from various formats (CryptoKey, JWK, Uint8Array).
 *
 * - If `key` is a CryptoKey, it's returned directly.
 * - If `key` is a Uint8Array, it's returned directly.
 * - If `key` is a JWK_oct (symmetric key with 'k'), the raw key bytes are returned as Uint8Array.
 * - If `key` is any other JWK type (asymmetric), the `alg` parameter is required, and a CryptoKey is returned.
 *
 * @param key The key to import.
 * @param alg The algorithm hint, required when importing asymmetric JWKs.
 * @returns A Promise resolving to the imported key as CryptoKey or Uint8Array.
 */
declare function importKey(key: string): Promise<Uint8Array>;
declare function importKey(key: Uint8Array): Promise<Uint8Array>;
declare function importKey(key: CryptoKey): Promise<CryptoKey>;
declare function importKey(key: JWK_oct): Promise<Uint8Array>;
declare function importKey(key: CryptoKey | JWK | Uint8Array | string, alg?: string): Promise<CryptoKey | Uint8Array>;
/**
 * Exports a CryptoKey to a JWK (JSON Web Key) format.
 *
 * @param key The CryptoKey to export.
 * @param jwk Optional partial JWK to merge with the exported key, allowing overrides.
 * @returns A Promise resolving to the exported JWK.
 */
declare function exportKey(key: CryptoKey, jwk?: Partial<JWK>): Promise<JWK>;
/**
 * Wraps a Content Encryption Key (CEK) using the specified algorithm and wrapping key.
 *
 * @param alg The JWA key management algorithm (e.g., "A128KW", "RSA-OAEP", "PBES2-HS256+A128KW").
 * @param keyToWrap The key to be wrapped (CEK), typically a symmetric key as Uint8Array or CryptoKey.
 * @param wrappingKey The key used to wrap the CEK (CryptoKey, JWK, or password string/Uint8Array for PBES2).
 * @param options Additional options required by certain algorithms (e.g., p2s, p2c for PBES2).
 * @returns A Promise resolving to an object containing the wrapped key and any necessary parameters (iv, tag, epk, etc.).
 */
declare function wrapKey(alg: KeyManagementAlgorithm, keyToWrap: CryptoKey | Uint8Array, wrappingKey: CryptoKey | JWK | string | Uint8Array, options?: WrapKeyOptions): Promise<WrapKeyResult>;
/**
 * Unwraps a Content Encryption Key (CEK) using the specified algorithm and unwrapping key.
 *
 * @param alg The JWA key management algorithm (e.g., "A128KW", "RSA-OAEP", "PBES2-HS256+A128KW").
 * @param wrappedKey The wrapped key (ciphertext) as Uint8Array.
 * @param unwrappingKey The key used to unwrap the CEK (CryptoKey, JWK, or password string/Uint8Array for PBES2).
 * @param options Additional options required by certain algorithms (e.g., iv, tag, p2s, p2c, epk).
 * @returns A Promise resolving to the unwrapped key (CEK) as a CryptoKey or Uint8Array.
 */
declare function unwrapKey<T extends boolean | undefined = undefined>(alg: KeyManagementAlgorithm, wrappedKey: Uint8Array, unwrappingKey: CryptoKey | JWK | string | Uint8Array, options?: UnwrapKeyOptions & {
    returnAs?: T;
}): Promise<T extends false ? Uint8Array : CryptoKey>;
/**
 * Imports a key from a PEM-encoded string and converts it to a JWK.
 *
 * @param pem The PEM-encoded string.
 * @param pemType The type of PEM encoding ('pkcs8' for private keys, 'spki' for public keys, 'x509' for certificates).
 * @param alg The JWA algorithm identifier. This is crucial for `crypto.subtle.importKey`
 *            to understand the key's intended algorithm and for setting the 'alg' field in the resulting JWK.
 * @param importOptions Options for the PEM import process (e.g., extractable for the CryptoKey).
 * @param jwkExtras Additional properties to merge into the resulting JWK.
 * @returns A Promise resolving to the imported key as a JWK.
 */
declare function importJWKFromPEM(pem: string, pemType: "pkcs8" | "spki" | "x509", alg: JWKPEMAlgorithm, importOptions?: KeyImportOptions, jwkExtras?: Partial<JWK>): Promise<JWK>;
/**
 * Exports a JWK to a PEM-encoded string.
 *
 * @param jwk The JWK to export.
 * @param pemFormat The desired PEM format ('pkcs8' for private keys, 'spki' for public keys).
 * @param algForCryptoKeyImport If the JWK does not have an 'alg' property, this algorithm hint is
 *                              required to correctly convert it to a CryptoKey first.
 * @returns A Promise resolving to the PEM-encoded key string.
 */
declare function exportJWKToPEM(jwk: JWK, pemFormat: "pkcs8" | "spki", algForCryptoKeyImport?: JWKPEMAlgorithm): Promise<string>;
/**
 * Retrieves a JWK (JSON Web Key) from a JWKSet based on a key ID (kid) or
 * properties from a JOSE Header.
 *
 * @param jwkSet The JWKSet to search within.
 * @param kidOrProtectedHeader Either a string representing the 'kid' (Key ID)
 *        or an object representing the JOSE Protected Header. If an object is
 *        provided, it must contain a 'kid' property. It can optionally contain
 *        'alg' (Algorithm) and 'kty' (Key Type) to further refine the search.
 * @returns The matching JWK from the set.
 * @throws TypeError if `jwkSet` is invalid or `kidOrProtectedHeader` is not a
 *         string or a valid header object.
 * @throws Error if no key is found matching the provided criteria, or if the
 *         header object is missing the 'kid' property.
 */
declare function getJWKFromSet(jwkSet: JWKSet, kidOrProtectedHeader: string | (JoseHeaderParameters & {
    alg?: string;
    kty?: string;
})): JWK;

declare const jwk_CompositeKey: typeof CompositeKey;
declare const jwk_ContentEncryptionAlgorithm: typeof ContentEncryptionAlgorithm;
declare const jwk_DeriveKeyOptions: typeof DeriveKeyOptions;
declare const jwk_DeriveKeyReturn: typeof DeriveKeyReturn;
declare const jwk_GenerateKeyAlgorithm: typeof GenerateKeyAlgorithm;
declare const jwk_GenerateKeyOptions: typeof GenerateKeyOptions;
declare const jwk_GenerateKeyReturn: typeof GenerateKeyReturn;
declare const jwk_JWK: typeof JWK;
declare const jwk_JWKAlgorithm: typeof JWKAlgorithm;
declare const jwk_JWKPEMAlgorithm: typeof JWKPEMAlgorithm;
declare const jwk_JWKParameters: typeof JWKParameters;
declare const jwk_JWKSet: typeof JWKSet;
declare const jwk_JWK_AES_CBC_HMAC: typeof JWK_AES_CBC_HMAC;
declare const jwk_JWK_AES_GCM: typeof JWK_AES_GCM;
declare const jwk_JWK_AES_GCM_KW: typeof JWK_AES_GCM_KW;
declare const jwk_JWK_AES_KW: typeof JWK_AES_KW;
declare const jwk_JWK_Asymmetric_Algorithm: typeof JWK_Asymmetric_Algorithm;
declare const jwk_JWK_EC: typeof JWK_EC;
declare const jwk_JWK_ECDH_ES: typeof JWK_ECDH_ES;
declare const jwk_JWK_ECDSA: typeof JWK_ECDSA;
declare const jwk_JWK_EC_Private: typeof JWK_EC_Private;
declare const jwk_JWK_EC_Public: typeof JWK_EC_Public;
declare const jwk_JWK_ED: typeof JWK_ED;
declare const jwk_JWK_HMAC: typeof JWK_HMAC;
declare const jwk_JWK_PBES2: typeof JWK_PBES2;
declare const jwk_JWK_RSA: typeof JWK_RSA;
declare const jwk_JWK_RSA_ENC: typeof JWK_RSA_ENC;
declare const jwk_JWK_RSA_PSS: typeof JWK_RSA_PSS;
declare const jwk_JWK_RSA_Private: typeof JWK_RSA_Private;
declare const jwk_JWK_RSA_Public: typeof JWK_RSA_Public;
declare const jwk_JWK_RSA_SIGN: typeof JWK_RSA_SIGN;
declare const jwk_JWK_Symmetric_Algorithm: typeof JWK_Symmetric_Algorithm;
declare const jwk_JWK_oct: typeof JWK_oct;
declare const jwk_KeyManagementAlgorithm: typeof KeyManagementAlgorithm;
declare const jwk_UnwrapKeyOptions: typeof UnwrapKeyOptions;
declare const jwk_WrapKeyOptions: typeof WrapKeyOptions;
declare const jwk_WrapKeyResult: typeof WrapKeyResult;
declare const jwk_deriveKeyFromPassword: typeof deriveKeyFromPassword;
declare const jwk_exportJWKToPEM: typeof exportJWKToPEM;
declare const jwk_exportKey: typeof exportKey;
declare const jwk_generateKey: typeof generateKey;
declare const jwk_getJWKFromSet: typeof getJWKFromSet;
declare const jwk_importJWKFromPEM: typeof importJWKFromPEM;
declare const jwk_importKey: typeof importKey;
declare const jwk_unwrapKey: typeof unwrapKey;
declare const jwk_wrapKey: typeof wrapKey;
declare namespace jwk {
  export {
    jwk_CompositeKey as CompositeKey,
    jwk_ContentEncryptionAlgorithm as ContentEncryptionAlgorithm,
    jwk_DeriveKeyOptions as DeriveKeyOptions,
    jwk_DeriveKeyReturn as DeriveKeyReturn,
    jwk_GenerateKeyAlgorithm as GenerateKeyAlgorithm,
    jwk_GenerateKeyOptions as GenerateKeyOptions,
    jwk_GenerateKeyReturn as GenerateKeyReturn,
    jwk_JWK as JWK,
    jwk_JWKAlgorithm as JWKAlgorithm,
    jwk_JWKPEMAlgorithm as JWKPEMAlgorithm,
    jwk_JWKParameters as JWKParameters,
    jwk_JWKSet as JWKSet,
    jwk_JWK_AES_CBC_HMAC as JWK_AES_CBC_HMAC,
    jwk_JWK_AES_GCM as JWK_AES_GCM,
    jwk_JWK_AES_GCM_KW as JWK_AES_GCM_KW,
    jwk_JWK_AES_KW as JWK_AES_KW,
    jwk_JWK_Asymmetric_Algorithm as JWK_Asymmetric_Algorithm,
    jwk_JWK_EC as JWK_EC,
    jwk_JWK_ECDH_ES as JWK_ECDH_ES,
    jwk_JWK_ECDSA as JWK_ECDSA,
    jwk_JWK_EC_Private as JWK_EC_Private,
    jwk_JWK_EC_Public as JWK_EC_Public,
    jwk_JWK_ED as JWK_ED,
    jwk_JWK_HMAC as JWK_HMAC,
    jwk_JWK_PBES2 as JWK_PBES2,
    jwk_JWK_RSA as JWK_RSA,
    jwk_JWK_RSA_ENC as JWK_RSA_ENC,
    jwk_JWK_RSA_PSS as JWK_RSA_PSS,
    jwk_JWK_RSA_Private as JWK_RSA_Private,
    jwk_JWK_RSA_Public as JWK_RSA_Public,
    jwk_JWK_RSA_SIGN as JWK_RSA_SIGN,
    jwk_JWK_Symmetric_Algorithm as JWK_Symmetric_Algorithm,
    jwk_JWK_oct as JWK_oct,
    jwk_KeyManagementAlgorithm as KeyManagementAlgorithm,
    jwk_UnwrapKeyOptions as UnwrapKeyOptions,
    jwk_WrapKeyOptions as WrapKeyOptions,
    jwk_WrapKeyResult as WrapKeyResult,
    jwk_deriveKeyFromPassword as deriveKeyFromPassword,
    jwk_exportJWKToPEM as exportJWKToPEM,
    jwk_exportKey as exportKey,
    jwk_generateKey as generateKey,
    jwk_getJWKFromSet as getJWKFromSet,
    jwk_importJWKFromPEM as importJWKFromPEM,
    jwk_importKey as importKey,
    jwk_unwrapKey as unwrapKey,
    jwk_wrapKey as wrapKey,
  };
}

export { importJWKFromPEM as a, exportJWKToPEM as b, getJWKFromSet as c, deriveKeyFromPassword as d, exportKey as e, generateKey as g, importKey as i, jwk as j, unwrapKey as u, wrapKey as w };
