import { o as JWK_HMAC, p as JWK_RSA_SIGN, q as JWK_RSA_PSS, r as JWK_ECDSA, m as JWK, n as JWKSet } from './unjwt.D9da8HQF.mjs';
import { J as JoseHeaderParameters, a as JWTClaims } from './unjwt.BvIkEGGz.mjs';

type JWS_SIGN_EXTRA = "Ed25519" | "EdDSA";
/** JWS Signing Algorithm Identifier. */
type JWSAlgorithm = JWK_HMAC | JWK_RSA_SIGN | JWK_RSA_PSS | JWK_ECDSA | JWS_SIGN_EXTRA;
/** Recognized JWS Header Parameters, any other Header Members may also be present. */
interface JWSHeaderParameters extends JoseHeaderParameters {
    /** JWS "alg" (Algorithm) Header Parameter */
    alg?: JWSAlgorithm;
    /**
     * This JWS Extension Header Parameter modifies the JWS Payload representation and the JWS Signing
     * Input computation as per {@link https://www.rfc-editor.org/rfc/rfc7797 RFC7797}.
     * If `false`, the payload is used directly without Base64URL encoding.
     * Defaults to `true`.
     */
    b64?: boolean;
}
/** JWS Protected Header */
interface JWSProtectedHeader extends JWSHeaderParameters {
    alg: JWSAlgorithm;
}
/**
 * JWS (JSON Web Signature) options for signing
 */
interface JWSSignOptions {
    /** The JWS Algorithm to use. Must be provided. */
    alg?: JWSAlgorithm;
    /**
     * Additional protected header parameters. `alg` is automatically included.
     * `typ` defaults to "JWT" if not provided.
     */
    protectedHeader?: JWSHeaderParameters;
    /** Date to use when comparing NumericDate claims, defaults to `new Date()`. */
    currentDate?: Date;
    /**
     * Time at which the JWS should expire, if no `exp` was already provided.
     */
    expiresIn?: number;
}
/** Result of JWS verification */
interface JWSVerifyResult<T extends JWTClaims | Uint8Array | string = JWTClaims | Uint8Array | string> {
    /** The decoded and verified payload. */
    payload: T;
    /** The JWS Protected Header. */
    protectedHeader: JWSProtectedHeader;
}
/** Options for JWS verification */
interface JWSVerifyOptions {
    /** List of allowed algorithms. If provided, the JWS `alg` must be in this list. */
    algorithms?: JWSAlgorithm[];
    /** If true, forces the payload to be returned as a Uint8Array, otherwise type is inferred. */
    forceUint8Array?: boolean;
    /**
     * Expected JWT "aud" (Audience) Claim value(s).
     *
     * This option makes the JWT "aud" (Audience) Claim presence required.
     */
    audience?: string | string[];
    /**
     * Clock skew tolerance (in seconds) for validating JWT claims.
     *
     * Used when validating the JWT "nbf" (Not Before) and "exp" (Expiration Time) claims, and when
     * validating the "iat" (Issued At) claim if the {@link maxTokenAge `maxTokenAge` option} is set.
     */
    clockTolerance?: number;
    /**
     * Expected JWT "iss" (Issuer) Claim value(s).
     *
     * This option makes the JWT "iss" (Issuer) Claim presence required.
     */
    issuer?: string | string[];
    /**
     * Maximum time elapsed (in seconds) from the JWT "iat" (Issued At) Claim value.
     *
     * This option makes the JWT "iat" (Issued At) Claim presence required.
     */
    maxTokenAge?: number;
    /**
     * Expected JWT "sub" (Subject) Claim value.
     *
     * This option makes the JWT "sub" (Subject) Claim presence required.
     */
    subject?: string;
    /**
     * Expected JWT "typ" (Type) Header Parameter value.
     *
     * This option makes the JWT "typ" (Type) Header Parameter presence required.
     */
    typ?: string;
    /** Date to use when comparing NumericDate claims, defaults to `new Date()`. */
    currentDate?: Date;
    /** List of critical header parameters that must be understood and processed. */
    requiredHeaders?: string[];
    /**
     * Array of required Claim Names that must be present in the JWT Claims Set. Default is that: if
     * the {@link issuer `issuer` option} is set, then JWT "iss" (Issuer) Claim must be present; if the
     * {@link audience `audience` option} is set, then JWT "aud" (Audience) Claim must be present; if
     * the {@link subject `subject` option} is set, then JWT "sub" (Subject) Claim must be present; if
     * the {@link maxTokenAge `maxTokenAge` option} is set, then JWT "iat" (Issued At) Claim must be
     * present.
     */
    requiredClaims?: string[];
    /**
     * List of critical header parameters that must be understood and processed.
     * @deprecated use {@link requiredHeaders `recognizedHeaders` option} instead.
     */
    critical?: string[];
}
/**
 * Key lookup function for JWS verification.
 * @param header The JWS Protected Header.
 * @param token The JWS token.
 * @returns The key material (CryptoKey, JWK, or raw Uint8Array) or a Promise resolving to it.
 */
type JWSKeyLookupFunction = (header: JWSProtectedHeader, token: string) => CryptoKey | JWK | JWKSet | Uint8Array | Promise<CryptoKey | JWK | JWKSet | Uint8Array>;

/**
 * Creates a JWS (JSON Web Signature) in Compact Serialization format.
 *
 * @param payload The payload to sign. Can be a string, Uint8Array, or an object (which will be JSON stringified).
 * @param key The signing key (CryptoKey, JWK, or raw symmetric key as Uint8Array).
 * @param options Signing options, including the algorithm (`alg`) and protected header parameters.
 * @returns A Promise resolving to the JWS Compact Serialization string.
 */
declare function sign(payload: JWTClaims, key: JWK, options?: JWSSignOptions): Promise<string>;
declare function sign(payload: string | Uint8Array | Record<string, any>, key: JWK, options?: JWSSignOptions): Promise<string>;
declare function sign(payload: JWTClaims, key: CryptoKey, options: JWSSignOptions & {
    alg: JWSAlgorithm;
}): Promise<string>;
declare function sign(payload: string | Uint8Array | Record<string, any>, key: CryptoKey, options: JWSSignOptions & {
    alg: JWSAlgorithm;
}): Promise<string>;
declare function sign(payload: JWTClaims, key: Uint8Array, options: JWSSignOptions & {
    alg: JWSAlgorithm;
}): Promise<string>;
declare function sign(payload: string | Uint8Array | Record<string, any>, key: Uint8Array, options: JWSSignOptions & {
    alg: JWSAlgorithm;
}): Promise<string>;
declare function sign(payload: string | Uint8Array | Record<string, any>, key: CryptoKey | JWK | Uint8Array, options: JWSSignOptions & {
    alg: JWSAlgorithm;
}): Promise<string>;
/**
 * Verifies a JWS (JSON Web Signature) in Compact Serialization format.
 *
 * @param jws The JWS Compact Serialization string.
 * @param key The verification key (CryptoKey, JWK, JWKSet, raw symmetric key as Uint8Array, or a function resolving the key or set).
 * @param options Verification options, such as allowed algorithms.
 * @returns A Promise resolving to an object containing the verified payload and protected header.
 * @throws If the JWS is invalid, signature verification fails, or options are not met.
 */
declare function verify<T extends JWTClaims | Uint8Array | string>(jws: string, key: CryptoKey | JWK | JWKSet | Uint8Array | JWSKeyLookupFunction, options?: JWSVerifyOptions): Promise<JWSVerifyResult<T>>;
declare function verify(jws: string, key: CryptoKey | JWK | JWKSet | Uint8Array | JWSKeyLookupFunction, options: JWSVerifyOptions & {
    forceUint8Array: true;
}): Promise<JWSVerifyResult<Uint8Array>>;

type jws_JWSAlgorithm = JWSAlgorithm;
type jws_JWSHeaderParameters = JWSHeaderParameters;
type jws_JWSKeyLookupFunction = JWSKeyLookupFunction;
type jws_JWSProtectedHeader = JWSProtectedHeader;
type jws_JWSSignOptions = JWSSignOptions;
type jws_JWSVerifyOptions = JWSVerifyOptions;
type jws_JWSVerifyResult<T extends JWTClaims | Uint8Array | string = JWTClaims | Uint8Array | string> = JWSVerifyResult<T>;
declare const jws_sign: typeof sign;
declare const jws_verify: typeof verify;
declare namespace jws {
  export { jws_sign as sign, jws_verify as verify };
  export type { jws_JWSAlgorithm as JWSAlgorithm, jws_JWSHeaderParameters as JWSHeaderParameters, jws_JWSKeyLookupFunction as JWSKeyLookupFunction, jws_JWSProtectedHeader as JWSProtectedHeader, jws_JWSSignOptions as JWSSignOptions, jws_JWSVerifyOptions as JWSVerifyOptions, jws_JWSVerifyResult as JWSVerifyResult };
}

export { jws as j, sign as s, verify as v };
export type { JWSAlgorithm as J, JWSHeaderParameters as a, JWSProtectedHeader as b, JWSSignOptions as c, JWSVerifyResult as d, JWSVerifyOptions as e, JWSKeyLookupFunction as f };
